import {
	btn,
	combobox,
	comp,
	datefield,
	DateInterval,
	DateTime,
	DisplayField,
	displayfield,
	fieldset,
	form,
	Format,
	NumberField,
	numberfield,
	t,
	tbar,
	Window
} from "@intermesh/goui";

import {RegistrationTable} from "@intermesh/business/registration";
import {client, jmapds, router} from "@intermesh/groupoffice-core";

export class InvoiceWindow extends Window {
	private invoiceAmountField!: NumberField;
	private invoicePercentageField!: NumberField;
	private registrationTable: RegistrationTable;
	private billHoursAmountField: DisplayField;
	private billHoursField: DisplayField;
	constructor(invoice:any, projectData:any) {
		super();

		this.width  = 700;
		this.height = 800;

		this.title = t("Create invoice");

		const outstanding = invoice.subtotalPrice - projectData.invoicedAmount + projectData.billedFee;

		this.items.add(
			form({flex: 1, cls: "vbox", handler: form1 => {
				const data = form1.value;

						client.jmap("FinanceDocument/invoiceProject", {
							bookId: data.bookId,
							percentage: data.invoicePercentage,
							documentId: projectData.documentId,
							registrationIds: this.registrationTable.store.data.map(r => r.id)
						}).then(invoice => {
							router.goto("FinanceDocument/" + invoice.id);
							this.close();
						}).catch(reason => {
							void Window.error(reason);
						})

					}},

				comp({ cls: "hbox"},

					fieldset({legend: t("Quote"), cls:"label-align-left vbox", flex: 1},

						displayfield({
							name: "number",
							label: t("Number"),
							value: invoice.number
						}),

						displayfield({
							name: "total",
							cls: "right",
							label: t("Total"),
							value: invoice.subtotalPrice,
							renderer: (v) => Format.money(v)
						}),

						displayfield({
							name: "invoicedAmount",
							cls: "right",
							label: t("Invoiced"),
							value: -1 * (projectData.invoicedAmount - projectData.billedFee),
							renderer: (v) => Format.money(v)
						}),

						displayfield({
							name: "outstanding",
							cls: "right border-top",
							label: t("Outstanding"),
							value: outstanding,
							renderer: (v) => Format.money(v)
						})
					),


					fieldset({legend: t("Hours"), flex: 1},

						comp({cls: "vbox label-align-left", flex: 1},
							displayfield({
								label: t("Billable"),
								cls: "right",
								value: projectData.billableFee,
								renderer: (v) => Format.money(v)
							}),

							displayfield({
								label: t("Billed"),
								cls: "right",
								value: projectData.billedFee,
								renderer: (v) => Format.money(-v)
							}),

							displayfield({
								cls: "right",
								label: t("Billed (h)"),
								value: projectData.billedMinutes,

								renderer: (v) => {
									const d = DateInterval.createFromFormat(v, "j")!;
									return d.format("h:I");
								}
							}),
						),
					),
				),

				fieldset({legend: "New"},

					comp({cls: "flow"},

						comp({cls: "flow", flex: 1},
							combobox({
								name: "bookId",
								label: t("Book"),
								dataSource: jmapds("FinanceBook"),
								storeConfig: {
									filters: {
										default: {
											type: "salesinvoice",
											businessId: projectData.businessId
										}
									}
								},
								listeners: {
									render: async comp1 => {
										const r = await comp1.list.store.load();
										if(r.length)
											comp1.value = r[0].id;
									}
								}
							}),
							comp({cls: "group"},

								this.invoiceAmountField = numberfield({
									name: "invoiceAmount",
									label: t("Amount"),
									decimals: 2,
									value: outstanding,
									min: 0,
									max: outstanding,
									listeners: {
										change: (field, newValue) => {
											if(field.isValid())
												this.invoicePercentageField.value = newValue * 100 / invoice.subtotalPrice ;
										}
									}
								}),
								this.invoicePercentageField = numberfield({
									name: "invoicePercentage",
									label: t("Percentage"),
									decimals: 0,
									min: 0,
									max: 100,
									value: outstanding * 100 / invoice.subtotalPrice ,
									listeners: {
										change: (field, newValue) => {
											if(field.isValid())
												this.invoiceAmountField.value = invoice.subtotalPrice * newValue / 100;
										}
									}
								})
							)
						),
						comp({flex: 1, cls: "flow"},

							datefield({
								name: "until",
								label: t("Bill hours until"),
								value: (new DateTime()).format("Y-m-d"),
								listeners: {
									change: (field, newValue, oldValue) => {
										void this.registrationTable.store
											.setFilter("until", {startDate: "< " +newValue})
											.load();
									}
								}
							}),

							comp({flex: 1, cls: "vbox label-align-left"},
							this.billHoursField = displayfield({
									cls: "right",
									label: t("Hours"),
									renderer:(v) => {
										const d = DateInterval.createFromFormat(v, "j")!;
										return d.format("h:I");
									}
								}),

								this.billHoursAmountField = displayfield({
									cls: "right",
									label: t("Amount"),
									value: projectData.billedFee,
									renderer: (v) => Format.money(v)
								}),
							)
						)
					)
				),

				comp({flex: 1, cls: "scroll bg-lowest"},
				this.registrationTable = new RegistrationTable()
				),

				tbar({},
					"->",
					btn({
						text: t("Create invoice"),
						type: "submit"
					})
				)
			)
		);


		this.registrationTable.store.on("datachanged", collection => {
			let totalMinutes = 0, totalBillableFee=0;
			collection.forEach((rec) => {
				totalMinutes += rec.bookedMinutes
				if(rec.hourlyRevenue) {
					totalBillableFee += rec.hourlyRevenue / 60 * rec.bookedMinutes;
				}
			})

			this.billHoursAmountField.value  = totalBillableFee;
			this.billHoursField.value = totalMinutes;
		})


		void this.registrationTable.store
			.setFilter( "defaults",{
				projectId: projectData.projectId,
				invoiced: false,
				billable: true
			})
			.setFilter("until", {startDate: "< " +(new DateTime()).format("Y-m-d")})
			.load();

	}
}