<?php
namespace go\modules\community\calendar\model;

use go\core\acl\model\AclOwnerEntity;
use go\core\db\Criteria;
use go\core\db\DbException;
use go\core\model\Principal;
use go\core\orm\exception\SaveException;
use go\core\orm\Filters;
use go\core\orm\Mapping;
use go\core\orm\PrincipalTrait;
use go\core\orm\Query;

/**
 * Calendar entity
 *
 */
class Calendar extends AclOwnerEntity {

	use PrincipalTrait {
		queryMissingPrincipals as protected traitMissingPrincipals;
	}

	/* Include In Availability */
	const All = 'all';
	const Attending = 'attending';
	const None = 'none';

	const UserProperties = ['color', 'sortOrder', 'isVisible', 'isSubscribed'];

	public $id;
	/** @var string The user-visible name of the calendar */
	public $name;
	public $description;
	/** @var string Any valid CSS color value. The color to be used when displaying events associated with the calendar */
	public $color;
	/** @var int uint32 Defines the sort order of calendars when presented in the client’s UI, so it is consistent between devices */
	public $sortOrder = 0;
	/** @var bool Has the user indicated they wish to see this Calendar in their client */
	public $isSubscribed;
	/** @var bool Should the calendar’s events be displayed to the user at the moment? */
	public $isVisible = true; // per user
	/**
	 * @var string (default: all) Should the calendar’s events be used as part of availability calculation?
	 * This MUST be one of:
	 *	- all: all events are considered.
	 *	- attending: events the user is a confirmed or tentative participant of are considered.
	 *	- none: all events are ignored (but may be considered if also in another calendar).
	 */
	public $includeInAvailability = self::All;

	/** @var ?string default for event. If NULL client will use the Users default timeZone  */
	public $timeZone;

	protected $defaultColor;

	public $defaultAlertsWithTime;
	public $defaultAlertsWithoutTime;
	public $ownerId;
	public $createdBy;

	public $groupId;
	protected $highestItemModSeq;

	protected static function defineMapping(): Mapping
	{
		return parent::defineMapping()
			->addTable("calendar_calendar")
			->addUserTable('calendar_calendar_user', 'caluser',['id' => 'id'], self::UserProperties)
			->addMap('defaultAlertsWithTime', DefaultAlert::class,  ['id'=>'fk'])
			->addMap('defaultAlertsWithoutTime', DefaultAlertWT::class,  ['id'=>'fk']);
	}

	protected static function textFilterColumns(): array
	{
		return ['name'];
	}

	/** @return int */
	public static function fetchDefault($scheduleId) {
		/** @var Preferences $pref */
		$pref = go()->getAuthState()->getUser(['calendarPreferences'])->calendarPreferences;
		if(!empty($pref->defaultCalendarId)) {
			return $pref->defaultCalendarId;
		}
		// If default preference is empty use the first owned calendar
		return self::find()->selectSingleValue('calendar_calendar.id')
			->join('core_user', 'u', 'u.id = calendar_calendar.ownerId')
			->where(['u.email' => $scheduleId])
			->andWhere(['groupId'=>null])
			->orderBy(['sortOrder'=>'ASC'])
			->single();
	}

	public function getColor() {
		return $this->color ?? $this->defaultColor;
	}
	public function setColor($value) {
		$this->color = $value;
	}


	protected static function defineFilters(): Filters
	{
		return parent::defineFilters()->add('isSubscribed', function(Criteria $criteria, $value, Query $query) {
			$criteria->where('isSubscribed','=', $value);
				if($value === false) {
					$criteria->orWhere('isSubscribed', 'IS', null);
				}
		})->add('isResource', function(Criteria $criteria, $value, Query $query) {
			$criteria->where('groupId',$value?'IS NOT':'IS', null);
		})->add('groupId', function(Criteria $criteria, $value, Query $query) {
			$criteria->where('groupId','=', $value);
		})->add('davaccountId', function(Criteria $criteria, $value, Query $query) {
			//$criteria->where('davc.davaccountId','=', $value);
		});
	}

	/**
	 * @return int highest mod
	 */
	public function highestItemModSeq() {
		return $this->highestItemModSeq;
	}

	static function updateHighestModSeq(\go\core\db\Query|int|array $calendarId) {
		go()->getDbConnection()
			->update(self::getMapping()->getPrimaryTable()->getName(),
				['highestItemModSeq' => CalendarEvent::getState()],
				['id' => $calendarId]
			)->execute();
	}

	/**
	 *  per-user OR default = true ONLY IF current user is the owner
	 * @return bool
	 */
//	public function getIsSubscribed() {
//		return ($this->isSubscribed === NULL) ? $this->isOwner() :  $this->isSubscribed;
//	}

	public function isOwner() {
		return $this->ownerId === go()->getUserId();
	}

	public function isOwned() {
		return !empty($this->ownerId);
	}

	protected function internalSave(): bool
	{
		if($this->isNew()) {
			$this->isSubscribed = true; // auto subscribe the creator.
			$this->isVisible = true;
			$this->defaultColor = $this->color;
		} else if($this->ownerId === go()->getUserId() && !empty($this->color)) {
			$this->defaultColor = $this->color;
		}
		if(empty($this->color)) {
			$this->color = $this->defaultColor;
		}
		if(!empty($this->groupId) && empty($this->ownerId)) {
			$this->ownerId = ResourceGroup::find()
				->selectSingleValue('defaultOwnerId')
				->where('id', '=', $this->groupId)
				->single();
		}
		if($this->isModified('defaultAlertsWithTime')) {
			$this->updateEventAlerts($this->defaultAlertsWithTime);
		}
		if($this->isModified('defaultAlertsWithoutTime')) {
			$this->updateEventAlerts($this->defaultAlertsWithoutTime, false);
		}
		return parent::internalSave(); // TODO: Change the autogenerated stub
	}

	/**
	 * @param DefaultAlert[] $defaultAlerts
	 * @param bool $withTime
	 * @return void
	 * @throws \Exception
	 */
	private function updateEventAlerts($defaultAlerts, $withTime = true) {
		// find all future eventIds
		$ids = CalendarEvent::find(['id'])->filter(['after' => date('Y-m-d\TH:i:s')])
			->andWhere('showWithoutTime', '=', $withTime?0:1)
			->andWhere('useDefaultAlerts', '=', 1)
			->fetchMode(\PDO::FETCH_COLUMN, 0)->all();
		if(empty($ids)) return;

		$type=CalendarEvent::entityType()->getId();
		// delete there alerts
		\go\core\model\Alert::delete((new Query())->where('entityId', 'in', $ids)
			->andWhere('entityTypeId', '=', $type)
			->andWhere('userId', '=', go()->getUserId()));
		// create new ones
		$events = CalendarEvent::find()->where('id','IN', $ids);
		foreach($events as $event) {
			foreach($defaultAlerts as $dalert) {
				$dalert->schedule($event);
			}
		}
	}

	public function getMyRights() {
		$lvl = $this->getPermissionLevel();
		return [
			'mayReadFreeBusy' => $lvl >= 5,
			'mayReadItems' => $lvl >= 10,
			'mayUpdatePrivate' => $lvl >= 20, // per-user properties only
			'mayRSVP' => $lvl >= 25, // only own principal status
			'mayWriteOwn' => $lvl >= 30, // write only owned events
			'mayWriteAll' => $lvl >= 35,
			'mayAdmin' => $lvl >= 50,
			'mayDelete' => $lvl >= 35 && $this->isOwner(), // calendar itself
		];
	}

	protected function principalAttrs(): array {
		// isPrinicpal() dictates an owner exists
		$owner = Principal::findById($this->ownerId);
		return [
			'name'=>$this->name,
			'description' => $this->description ?? $owner->name ?? '',
			'timeZone' => $this->timeZone,
			'email' => $owner->email
		];
	}

	protected function isPrincipal()
	{
		return $this->groupId != null && $this->ownerId != null;
	}

	protected static function queryMissingPrincipals(int $offset = 0): Query {
		return self::traitMissingPrincipals($offset)->andWhere('groupId', 'IS NOT', NULL);
	}

	protected function isPrincipalModified() : bool
	{
		return $this->isModified(['name', 'description', 'timeZone', 'ownerId','groupId']);
	}

	protected function principalType(): string {
		return Principal::Resource;
	}
}
